==> printly-0.1.0.tar.gz: printly-0.1.0/printly/style.py <==
"""Define functions for applying color and font styles to text."""

import re
from typing import Dict, Generator, Optional, Tuple
from .types import Color, FontStyle

HEX_PREFIX: str = "#"
RGB_DELIMITER: str = ","
COMBINATOR: str = "+"
RESET_CODE: str = "\033[0m"

COLORS_RGB_MAP: Dict[Color, Tuple[int, int, int]] = {
    # Shades of Green
    "greenyellow": (173, 255, 47),
    "chartreuse": (127, 255, 0),
    "lawngreen": (124, 252, 0),
    "lime": (0, 255, 0),
    "limegreen": (50, 205, 50),
    "palegreen": (152, 251, 152),
    "lightgreen": (144, 238, 144),
    "mediumspringgreen": (0, 250, 154),
    "springgreen": (0, 255, 127),
    "mediumseagreen": (60, 179, 113),
    "seagreen": (46, 139, 87),
    "forestgreen": (34, 139, 34),
    "green": (0, 128, 0),
    "darkgreen": (0, 100, 0),
    "yellowgreen": (154, 205, 50),
    "olivedrab": (107, 142, 35),
    "olive": (128, 128, 0),
    "darkolivegreen": (85, 107, 47),
    "mediumaquamarine": (102, 205, 170),
    "darkseagreen": (143, 188, 143),
    "lightseagreen": (32, 178, 170),
    "darkcyan": (0, 139, 139),
    "teal": (0, 128, 128),
    # Shades of Blue
    "cyan": (0, 255, 255),
    "aqua": (0, 255, 255),
    "lightcyan": (224, 255, 255),
    "paleturquoise": (175, 238, 238),
    "aquamarine": (127, 255, 212),
    "turquoise": (64, 224, 208),
    "mediumturquoise": (72, 209, 204),
    "darkturquoise": (0, 206, 209),
    "cadetblue": (95, 158, 160),
    "steelblue": (70, 180, 130),
    "lightsteelblue": (176, 196, 222),
    "powderblue": (176, 224, 230),
    "lightblue": (173, 216, 230),
    "skyblue": (135, 206, 235),
    "lightskyblue": (135, 206, 250),
    "deepskyblue": (0, 191, 255),
    "dodgerblue": (30, 144, 255),
    "cornflowerblue": (100, 149, 237),
    "mediumslateblue": (123, 104, 238),
    "royalblue": (65, 105, 225),
    "blue": (0, 0, 255),
    "mediumblue": (0, 0, 205),
    "darkblue": (0, 0, 139),
    "navy": (0, 0, 128),
    "midnightblue": (25, 25, 112),
    # Shades of Red
    "indianred": (205, 92, 92),
    "lightcoral": (240, 128, 128),
    "salmon": (250, 128, 114),
    "darksalmon": (233, 150, 122),
    "crimson": (220, 20, 60),
    "red": (225, 0, 0),
    "firebrick": (178, 34, 34),
    "darkred": (139, 0, 0),
    # Shades of Orange
    "lightsalmon": (255, 160, 122),
    "coral": (255, 127, 80),
    "tomato": (255, 99, 71),
    "orangered": (255, 69, 0),
    "darkorange": (255, 140, 0),
    "orange": (255, 165, 0),
    # Shades of Yellow
    "gold": (255, 215, 0),
    "yellow": (255, 255, 0),
    "lightyellow": (255, 255, 224),
    "lemonchiffon": (255, 250, 205),
    "lightgoldenrodyellow": (250, 250, 210),
    "papayawhip": (255, 239, 213),
    "moccasin": (255, 228, 181),
    "peachpuff": (255, 218, 185),
    "palegoldenrod": (238, 232, 170),
    "khaki": (240, 230, 140),
    "darkkhaki": (189, 183, 107),
    # Shades of Purple
    "lavender": (230, 230, 250),
    "thistle": (216, 191, 216),
    "plum": (221, 160, 221),
    "violet": (238, 130, 238),
    "orchid": (218, 112, 214),
    "magenta": (255, 0, 255),
    "fuchsia": (255, 0, 255),
    "mediumorchid": (186, 85, 211),
    "mediumpurple": (147, 112, 219),
    "blueviolet": (138, 43, 226),
    "darkviolet": (148, 0, 211),
    "darkorchid": (153, 50, 204),
    "darkmagenta": (139, 0, 139),
    "purple": (128, 0, 128),
    "rebeccapurple": (102, 51, 153),
    "indigo": (75, 0, 130),
    "slateblue": (106, 90, 205),
    "darkslateblue": (72, 61, 139),
    # Shades of Pink
    "pink": (255, 192, 203),
    "lightpink": (255, 182, 193),
    "hotpink": (255, 105, 180),
    "deeppink": (255, 20, 147),
    "mediumvioletred": (199, 21, 133),
    "palevioletred": (219, 112, 147),
    # Shades of Brown
    "cornsilk": (255, 248, 220),
    "blanchedalmond": (255, 235, 205),
    "bisque": (255, 228, 196),
    "navajowhite": (255, 222, 173),
    "wheat": (245, 222, 179),
    "burlywood": (222, 184, 135),
    "tan": (210, 180, 140),
    "rosybrown": (188, 143, 143),
    "sandybrown": (244, 164, 96),
    "goldenrod": (218, 165, 35),
    "darkgoldenrod": (184, 134, 11),
    "peru": (205, 133, 63),
    "chocolate": (210, 105, 30),
    "saddlebrown": (139, 69, 19),
    "sienna": (160, 82, 45),
    "maroon": (128, 0, 0),
    "brown": (165, 42, 42),
    # Shades of Grey
    "gainsboro": (220, 220, 220),
    "lightgrey": (211, 211, 211),
    "lightgray": (211, 211, 211),
    "silver": (192, 192, 192),
    "darkgrey": (169, 169, 169),
    "darkgray": (169, 169, 169),
    "grey": (128, 128, 128),
    "gray": (128, 128, 128),
    "dimgrey": (105, 105, 105),
    "dimgray": (105, 105, 105),
    "lightslategrey": (119, 136, 153),
    "lightslategray": (119, 136, 153),
    "slategrey": (112, 128, 114),
    "slategray": (112, 128, 114),
    "darkslategrey": (49, 79, 79),
    "darkslategray": (49, 79, 79),
    "black": (0, 0, 0),
    # Shades of White
    "white": (255, 255, 255),
    "snow": (255, 250, 250),
    "honeydew": (240, 255, 240),
    "mintcream": (245, 255, 250),
    "azure": (240, 255, 255),
    "aliceblue": (240, 248, 255),
    "ghostwhite": (248, 248, 255),
    "whitesmoke": (245, 245, 245),
    "seashell": (255, 235, 248),
    "beige": (245, 245, 220),
    "oldlace": (253, 245, 230),
    "floralwhite": (255, 250, 240),
    "ivory": (255, 255, 240),
    "antiquewhite": (250, 235, 215),
    "linen": (250, 240, 230),
    "lavenderblush": (255, 240, 245),
    "mistyrose": (255, 228, 255),
}

FONT_STYLE_CODES: Dict[FontStyle, int] = {
    "bold": 1,
    "italic": 3,
    "underline": 4,
    "hide": 8,
    "strikethrough": 9,
    "double-underline": 21,
    "overline": 53,
}


def _convert_hex_color_to_rgb_values(color: str) -> Tuple[int, int, int]:
    """Converts a hex color code to an RGB tuple."""
    if not (match := re.match(r"^#([0-9a-fA-F]{3}|[0-9a-fA-F]{6})$", color)):
        raise ValueError(f"Invalid hex color '{color}'.")

    if len(hex_digits := match.group(1)) == 3:
        hex_digits = "".join(hex_digit * 2 for hex_digit in hex_digits)

    return tuple(int(hex_digits[i : i + 2], base=16) for i in range(0, 6, 2))  # type: ignore


def _convert_rgb_color_to_rgb_values(color: str) -> Tuple[int, int, int]:
    """Parses an RGB string into an RGB tuple."""
    try:
        rgb_values = tuple(map(int, color.split(RGB_DELIMITER)))
    except ValueError as exc:
        raise ValueError(f"Invalid RGB color '{color}'. Values must be integers.") from exc

    if len(rgb_values) != 3:
        raise ValueError(f"Invalid RGB color '{color}'. Must have exactly 3 values.")
    if not 0 <= max(rgb_values) <= 255:
        raise ValueError(f"Invalid RGB color '{color}'.  Values must be in range 0-255.")
    return rgb_values


def _convert_named_color_to_rgb_values(color: Color) -> Tuple[int, int, int]:
    """Retrieves the RGB tuple for a named color."""
    if rgb_values := COLORS_RGB_MAP.get(color):
        return rgb_values
    raise ValueError(f"Invalid color name '{color}'.")


def _get_rgb_values(color: Color) -> Tuple[int, int, int]:
    """Determines the RGB values for a given color input."""
    color = re.sub(r"(\s+|-+)", "", f"{color}").lower()
    if color.startswith(HEX_PREFIX):
        return _convert_hex_color_to_rgb_values(color)
    if RGB_DELIMITER in color:
        return _convert_rgb_color_to_rgb_values(color)
    if color.replace("-", "").isalpha():
        return _convert_named_color_to_rgb_values(color)
    raise ValueError(f"Unrecognized color '{color}'.")


def _get_fontstyle_codes(fontstyle: FontStyle) -> Generator[int]:
    """Retrieves font style codes for a given font style input."""
    fontstyle = re.sub(r"\s+", "", f"{fontstyle}").lower()
    for fontstyle in fontstyle.split(COMBINATOR):
        if fontstyle_code := FONT_STYLE_CODES.get(fontstyle):
            yield fontstyle_code
        else:
            raise ValueError(f"Unrecognized font style '{fontstyle}'.")


def style(
    text: str,
    fg: Optional[Color] = None,
    bg: Optional[Color] = None,
    fs: Optional[FontStyle] = None,
) -> str:
    """
    Applies foreground color, background color, and font style, to text.

    Args:
        fg (Color | None): Foreground color for the text. Defaults to `None`.
        bg (Color | None): Background color for the text. Defaults to `None`.
        fs (FontStyle | None): Font style(s) for the text. Defaults to `None`.

    Returns:
        str: Styled text.
    """
    if fg or bg or fs:
        fg_code = bg_code = fs_code = ""
        if fg:
            fg_code = f"\033[38;2;{';'.join(str(value) for value in _get_rgb_values(color=fg))}m"
        if bg:
            bg_code = f"\033[48;2;{';'.join(str(value) for value in _get_rgb_values(color=bg))}m"
        if fs:
            fs_code = "".join((f"\033[{code}m" for code in _get_fontstyle_codes(fontstyle=fs)))
        styles = fg_code + bg_code + fs_code
        styled_text = styles + text.replace("\n", RESET_CODE + "\n" + styles)
        styled_text += RESET_CODE if not styled_text.endswith(RESET_CODE) else ""
        return styled_text
    return text

==> printly-0.1.1.tar.gz: printly-0.1.1/printly/style.py <==
"""Define functions for applying color and font styles to text."""

import re
from typing import Dict, Generator, Optional, Tuple
from .types import Color, FontStyle

HEX_PREFIX: str = "#"
RGB_DELIMITER: str = ","
COMBINATOR: str = "+"
RESET_CODE: str = "\033[0m"

COLORS_RGB_MAP: Dict[Color, Tuple[int, int, int]] = {
    # Shades of Green
    "greenyellow": (173, 255, 47),
    "chartreuse": (127, 255, 0),
    "lawngreen": (124, 252, 0),
    "lime": (0, 255, 0),
    "limegreen": (50, 205, 50),
    "palegreen": (152, 251, 152),
    "lightgreen": (144, 238, 144),
    "mediumspringgreen": (0, 250, 154),
    "springgreen": (0, 255, 127),
    "mediumseagreen": (60, 179, 113),
    "seagreen": (46, 139, 87),
    "forestgreen": (34, 139, 34),
    "green": (0, 128, 0),
    "darkgreen": (0, 100, 0),
    "yellowgreen": (154, 205, 50),
    "olivedrab": (107, 142, 35),
    "olive": (128, 128, 0),
    "darkolivegreen": (85, 107, 47),
    "mediumaquamarine": (102, 205, 170),
    "darkseagreen": (143, 188, 143),
    "lightseagreen": (32, 178, 170),
    "darkcyan": (0, 139, 139),
    "teal": (0, 128, 128),
    # Shades of Blue
    "cyan": (0, 255, 255),
    "aqua": (0, 255, 255),
    "lightcyan": (224, 255, 255),
    "paleturquoise": (175, 238, 238),
    "aquamarine": (127, 255, 212),
    "turquoise": (64, 224, 208),
    "mediumturquoise": (72, 209, 204),
    "darkturquoise": (0, 206, 209),
    "cadetblue": (95, 158, 160),
    "steelblue": (70, 180, 130),
    "lightsteelblue": (176, 196, 222),
    "powderblue": (176, 224, 230),
    "lightblue": (173, 216, 230),
    "skyblue": (135, 206, 235),
    "lightskyblue": (135, 206, 250),
    "deepskyblue": (0, 191, 255),
    "dodgerblue": (30, 144, 255),
    "cornflowerblue": (100, 149, 237),
    "mediumslateblue": (123, 104, 238),
    "royalblue": (65, 105, 225),
    "blue": (0, 0, 255),
    "mediumblue": (0, 0, 205),
    "darkblue": (0, 0, 139),
    "navy": (0, 0, 128),
    "midnightblue": (25, 25, 112),
    # Shades of Red
    "indianred": (205, 92, 92),
    "lightcoral": (240, 128, 128),
    "salmon": (250, 128, 114),
    "darksalmon": (233, 150, 122),
    "crimson": (220, 20, 60),
    "red": (225, 0, 0),
    "firebrick": (178, 34, 34),
    "darkred": (139, 0, 0),
    # Shades of Orange
    "lightsalmon": (255, 160, 122),
    "coral": (255, 127, 80),
    "tomato": (255, 99, 71),
    "orangered": (255, 69, 0),
    "darkorange": (255, 140, 0),
    "orange": (255, 165, 0),
    # Shades of Yellow
    "gold": (255, 215, 0),
    "yellow": (255, 255, 0),
    "lightyellow": (255, 255, 224),
    "lemonchiffon": (255, 250, 205),
    "lightgoldenrodyellow": (250, 250, 210),
    "papayawhip": (255, 239, 213),
    "moccasin": (255, 228, 181),
    "peachpuff": (255, 218, 185),
    "palegoldenrod": (238, 232, 170),
    "khaki": (240, 230, 140),
    "darkkhaki": (189, 183, 107),
    # Shades of Purple
    "lavender": (230, 230, 250),
    "thistle": (216, 191, 216),
    "plum": (221, 160, 221),
    "violet": (238, 130, 238),
    "orchid": (218, 112, 214),
    "magenta": (255, 0, 255),
    "fuchsia": (255, 0, 255),
    "mediumorchid": (186, 85, 211),
    "mediumpurple": (147, 112, 219),
    "blueviolet": (138, 43, 226),
    "darkviolet": (148, 0, 211),
    "darkorchid": (153, 50, 204),
    "darkmagenta": (139, 0, 139),
    "purple": (128, 0, 128),
    "rebeccapurple": (102, 51, 153),
    "indigo": (75, 0, 130),
    "slateblue": (106, 90, 205),
    "darkslateblue": (72, 61, 139),
    # Shades of Pink
    "pink": (255, 192, 203),
    "lightpink": (255, 182, 193),
    "hotpink": (255, 105, 180),
    "deeppink": (255, 20, 147),
    "mediumvioletred": (199, 21, 133),
    "palevioletred": (219, 112, 147),
    # Shades of Brown
    "cornsilk": (255, 248, 220),
    "blanchedalmond": (255, 235, 205),
    "bisque": (255, 228, 196),
    "navajowhite": (255, 222, 173),
    "wheat": (245, 222, 179),
    "burlywood": (222, 184, 135),
    "tan": (210, 180, 140),
    "rosybrown": (188, 143, 143),
    "sandybrown": (244, 164, 96),
    "goldenrod": (218, 165, 35),
    "darkgoldenrod": (184, 134, 11),
    "peru": (205, 133, 63),
    "chocolate": (210, 105, 30),
    "saddlebrown": (139, 69, 19),
    "sienna": (160, 82, 45),
    "maroon": (128, 0, 0),
    "brown": (165, 42, 42),
    # Shades of Grey
    "gainsboro": (220, 220, 220),
    "lightgrey": (211, 211, 211),
    "lightgray": (211, 211, 211),
    "silver": (192, 192, 192),
    "darkgrey": (169, 169, 169),
    "darkgray": (169, 169, 169),
    "grey": (128, 128, 128),
    "gray": (128, 128, 128),
    "dimgrey": (105, 105, 105),
    "dimgray": (105, 105, 105),
    "lightslategrey": (119, 136, 153),
    "lightslategray": (119, 136, 153),
    "slategrey": (112, 128, 114),
    "slategray": (112, 128, 114),
    "darkslategrey": (49, 79, 79),
    "darkslategray": (49, 79, 79),
    "black": (0, 0, 0),
    # Shades of White
    "white": (255, 255, 255),
    "snow": (255, 250, 250),
    "honeydew": (240, 255, 240),
    "mintcream": (245, 255, 250),
    "azure": (240, 255, 255),
    "aliceblue": (240, 248, 255),
    "ghostwhite": (248, 248, 255),
    "whitesmoke": (245, 245, 245),
    "seashell": (255, 235, 248),
    "beige": (245, 245, 220),
    "oldlace": (253, 245, 230),
    "floralwhite": (255, 250, 240),
    "ivory": (255, 255, 240),
    "antiquewhite": (250, 235, 215),
    "linen": (250, 240, 230),
    "lavenderblush": (255, 240, 245),
    "mistyrose": (255, 228, 255),
}

FONT_STYLE_CODES: Dict[FontStyle, int] = {
    "bold": 1,
    "italic": 3,
    "underline": 4,
    "hide": 8,
    "strikethrough": 9,
    "double-underline": 21,
    "overline": 53,
}


def _convert_hex_color_to_rgb_values(color: str) -> Tuple[int, int, int]:
    """Converts a hex color code to an RGB tuple."""
    if not (match := re.match(r"^#([0-9a-fA-F]{3}|[0-9a-fA-F]{6})$", color)):
        raise ValueError(f"Invalid hex color '{color}'.")

    if len(hex_digits := match.group(1)) == 3:
        hex_digits = "".join(hex_digit * 2 for hex_digit in hex_digits)

    return tuple(int(hex_digits[i : i + 2], base=16) for i in range(0, 6, 2))  # type: ignore


def _convert_rgb_color_to_rgb_values(color: str) -> Tuple[int, int, int]:
    """Parses an RGB string into an RGB tuple."""
    try:
        rgb_values = tuple(map(int, color.split(RGB_DELIMITER)))
    except ValueError as exc:
        raise ValueError(f"Invalid RGB color '{color}'. Values must be integers.") from exc

    if len(rgb_values) != 3:
        raise ValueError(f"Invalid RGB color '{color}'. Must have exactly 3 values.")
    if not 0 <= max(rgb_values) <= 255:
        raise ValueError(f"Invalid RGB color '{color}'.  Values must be in range 0-255.")
    return rgb_values


def _convert_named_color_to_rgb_values(color: Color) -> Tuple[int, int, int]:
    """Retrieves the RGB tuple for a named color."""
    if rgb_values := COLORS_RGB_MAP.get(color):
        return rgb_values
    raise ValueError(f"Invalid color name '{color}'.")


def _get_rgb_values(color: Color) -> Tuple[int, int, int]:
    """Determines the RGB values for a given color input."""
    color = re.sub(r"(\s+|-+)", "", f"{color}").lower()
    if color.startswith(HEX_PREFIX):
        return _convert_hex_color_to_rgb_values(color)
    if RGB_DELIMITER in color:
        return _convert_rgb_color_to_rgb_values(color)
    if color.replace("-", "").isalpha():
        return _convert_named_color_to_rgb_values(color)
    raise ValueError(f"Unrecognized color '{color}'.")


def _get_fontstyle_codes(fontstyle: FontStyle) -> Generator[int, None, None]:
    """Retrieves font style codes for a given font style input."""
    fontstyle = re.sub(r"\s+", "", f"{fontstyle}").lower()
    for fontstyle in fontstyle.split(COMBINATOR):
        if not (fontstyle_code := FONT_STYLE_CODES.get(fontstyle)):
            raise ValueError(f"Unrecognized font style '{fontstyle}'.")
        yield fontstyle_code


def style(
    text: str,
    fg: Optional[Color] = None,
    bg: Optional[Color] = None,
    fs: Optional[FontStyle] = None,
) -> str:
    """
    Applies foreground color, background color, and font style, to text.

    Args:
        fg (Color | None): Foreground color for the text. Defaults to `None`.
        bg (Color | None): Background color for the text. Defaults to `None`.
        fs (FontStyle | None): Font style(s) for the text. Defaults to `None`.

    Returns:
        str: Styled text.
    """
    if fg or bg or fs:
        fg_code = bg_code = fs_code = ""
        if fg:
            fg_code = f"\033[38;2;{';'.join(str(value) for value in _get_rgb_values(color=fg))}m"
        if bg:
            bg_code = f"\033[48;2;{';'.join(str(value) for value in _get_rgb_values(color=bg))}m"
        if fs:
            fs_code = "".join((f"\033[{code}m" for code in _get_fontstyle_codes(fontstyle=fs)))
        styles = fg_code + bg_code + fs_code
        styled_text = styles + text.replace("\n", RESET_CODE + "\n" + styles)
        styled_text += RESET_CODE if not styled_text.endswith(RESET_CODE) else ""
        return styled_text
    return text
